@@ -8,6 +8,7 @@ module Network.OAuth.OAuth2.HttpClient (
   fetchAccessToken,
   fetchRefreshToken,
   doJSONPostRequest,
+  doFlexiblePostRequest,
   doSimplePostRequest,
 -- * AUTH requests
   authGetJSON,
@@ -15,10 +16,12 @@ module Network.OAuth.OAuth2.HttpClient (
   authGetBS',
   authPostJSON,
   authPostBS,
+  authPostBS',
   authRequest,
 -- * Utilities
   handleResponse,
   parseResponseJSON,
+  parseResponseFlexible,
   updateRequestHeaders,
   setMethod
 ) where
@@ -27,10 +30,12 @@ import           Control.Monad                 (liftM)
 import           Data.Aeson
 import qualified Data.ByteString.Char8         as BS
 import qualified Data.ByteString.Lazy.Char8    as BSL
+import qualified Data.HashMap.Strict           as HM (fromList)
 import           Data.Maybe
+import qualified Data.Text.Encoding            as T
 import           Network.HTTP.Conduit          hiding (withManager)
 import qualified Network.HTTP.Types            as HT
-
+import           Network.HTTP.Types.URI        (parseQuery)
 import           Network.OAuth.OAuth2.Internal
 
 --------------------------------------------------
@@ -42,7 +47,7 @@ fetchAccessToken :: Manager                          -- ^ HTTP connection manage
                    -> OAuth2                         -- ^ OAuth Data
                    -> BS.ByteString                  -- ^ Authentication code gained after authorization
                    -> IO (OAuth2Result AccessToken)  -- ^ Access Token
-fetchAccessToken manager oa code = doJSONPostRequest manager oa uri body
+fetchAccessToken manager oa code = doFlexiblePostRequest manager oa uri body
                            where (uri, body) = accessTokenUrl oa code
 
 
@@ -51,7 +56,7 @@ fetchRefreshToken :: Manager                         -- ^ HTTP connection manage
                      -> OAuth2                       -- ^ OAuth context
                      -> BS.ByteString                -- ^ refresh token gained after authorization
                      -> IO (OAuth2Result AccessToken)
-fetchRefreshToken manager oa rtoken = doJSONPostRequest manager oa uri body
+fetchRefreshToken manager oa rtoken = doFlexiblePostRequest manager oa uri body
                               where (uri, body) = refreshAccessTokenUrl oa rtoken
 
 
@@ -64,6 +69,15 @@ doJSONPostRequest :: FromJSON a
                   -> IO (OAuth2Result a)                 -- ^ Response as ByteString
 doJSONPostRequest manager oa uri body = liftM parseResponseJSON (doSimplePostRequest manager oa uri body)
 
+-- | Conduct post request and return response as JSON or Query String.
+doFlexiblePostRequest :: FromJSON a
+                         => Manager                             -- ^ HTTP connection manager.
+                         -> OAuth2                              -- ^ OAuth options
+                         -> URI                                 -- ^ The URL
+                         -> PostBody                            -- ^ request body
+                         -> IO (OAuth2Result a)                 -- ^ Response as ByteString
+doFlexiblePostRequest manager oa uri body = liftM parseResponseFlexible (doSimplePostRequest manager oa uri body)
+
 -- | Conduct post request.
 doSimplePostRequest :: Manager                              -- ^ HTTP connection manager.
                        -> OAuth2                            -- ^ OAuth options
@@ -131,6 +145,19 @@ authPostBS manager token url pb = do
         upHeaders = updateRequestHeaders (Just token) . setMethod HT.POST
         upReq = upHeaders . upBody
 
+-- | Conduct POST request with access token in the request body rather header
+authPostBS' :: Manager                             -- ^ HTTP connection manager.
+             -> AccessToken
+             -> URI                               -- ^ URL
+             -> PostBody
+             -> IO (OAuth2Result BSL.ByteString)  -- ^ Response as ByteString
+authPostBS' manager token url pb = do
+  req <- parseUrl $ BS.unpack url
+  authRequest req upReq manager
+  where upBody = urlEncodedBody (pb ++ accessTokenToParam token)
+        upHeaders = updateRequestHeaders Nothing . setMethod HT.POST
+        upReq = upHeaders . upBody
+
 -- |Send an HTTP request including the Authorization header with the specified
 --  access token.
 --
@@ -149,7 +176,7 @@ handleResponse :: Response BSL.ByteString -> OAuth2Result BSL.ByteString
 handleResponse rsp =
     if HT.statusIsSuccessful (responseStatus rsp)
         then Right $ responseBody rsp
-        else Left $ BSL.append "hoauth2.HttpClient.parseResponseJSON/Gaining token failed: " (responseBody rsp)
+        else Left $ BSL.append "hoauth2.HttpClient.handleResponse/Gaining token failed: " (responseBody rsp)
 
 -- | Parses a @OAuth2Result BSL.ByteString@ into @FromJSON a => a@
 parseResponseJSON :: FromJSON a
@@ -160,6 +187,29 @@ parseResponseJSON (Right b) = case decode b of
                             Nothing -> Left ("hoauth2.HttpClient.parseResponseJSON/Could not decode JSON: " `BSL.append` b)
                             Just x -> Right x
 
+-- | Parses a @OAuth2Result BSL.ByteString@ that contains not JSON but a Query String
+parseResponseString :: FromJSON a
+              => OAuth2Result BSL.ByteString
+              -> OAuth2Result a
+parseResponseString (Left b) = Left b
+parseResponseString (Right b) = case parseQuery $ BSL.toStrict b of
+                              [] -> Left errorMessage
+                              a -> case fromJSON $ queryToValue a of
+                                    Error _ -> Left errorMessage
+                                    Success x -> Right x
+  where
+    queryToValue = Object . HM.fromList . map paramToPair
+    paramToPair (k, mv) = (T.decodeUtf8 k, maybe Null (String . T.decodeUtf8) mv)
+    errorMessage = "hoauth2.HttpClient.parseResponseJSON/Could not decode JSON or URL: " `BSL.append` b
+
+-- | Try 'parseResponseJSON' and 'parseResponseString'
+parseResponseFlexible :: FromJSON a
+                         => OAuth2Result BSL.ByteString
+                         -> OAuth2Result a
+parseResponseFlexible r = case parseResponseJSON r of
+                           Left _ -> parseResponseString r
+                           x -> x
+
 -- | Set several header values:
 --   + userAgennt    : `hoauth2`
 --   + accept        : `application/json`
